if (typeof String.prototype.startsWith != 'function') {
  // see below for better implementation!
  String.prototype.startsWith = function (str){
    return this.indexOf(str) == 0;
  };
}

getAudioUrl = function(audioName) {
    url = "http://media.merriam-webster.com/soundc11/";
    if (audioName.startsWith("bix")) {
      url += "bix/";
    } else if (audioName.startsWith("gg")) {
      url += "gg/";
    } else if (/^\d+.*/.test(audioName)) {
      url += "number/";
    } else {
      url += audioName.charAt(0);
      url += '/';
    }
    url += audioName;
    url += ".wav";
    return url;
}

$(function() {
    FastClick.attach(document.body);
});

$(document).ready(function() {

  showDictionary();

  // Make a POS number tab
  //$(".posno").wrapInner('<div class="pos-tab" />');

  //Handle links
  $("a").each(function(index) {
    var $this = $(this);
    var href = $this.attr('href');

    if ($this.hasClass("photo")) {
      $this.removeClass("photo");
      $this.addClass("art-link");
    }

    if (href.search(/\/table\/dict\/.*/) != -1) {
      // convert table/dict hrefs to normal text
      $this.replaceWith($this.contents());
    } else if (href.indexOf('/') == 0) {
      if (href.indexOf('/art') == 0) {
        $this.attr('href', 'x-mw://art?link=' + encodeURIComponent('http://www.merriam-webster.com' + href));
      } else {
        $this.attr('href', 'http://www.merriam-webster.com' + href);
      }
    } else {
      // Add our own click handler
      $this.click(function(e) {
        var that = $(this);
        if (that.hasClass("photo")) {
            // Gallery link
            ;       // do nothing special since this is already an x-mw link
        } else if (that.hasClass("purchase")) {
            // Purchase link
            ;       // do nothing special since this is already an x-mw link
        } else {
          if (that.attr('href').search(/x\-mw\:\/\/.*/) != -1) {
            // A recognized internal x-mw:// href
            ;       // do nothing special since this is already an x-mw link
          } else {
            // disable normal handling of an A tag
            e.preventDefault();
            
            // Apply a temporary style to selected term for word click
            that.addClass("selected");

            // Callback to lookup the selected term.
            var word = that.attr('href').replace(/http.+definition\/([\w%\d]+)(\[\d+\])?/g, '$1');
            if (word && word.length >= that.text().length) {
              window.location.href = "x-mw://lookup/" + word;
            } else {
              window.location.href = "x-mw://lookup/" + that.text();
            }

            // After a small delay, unselect the term
            setTimeout(function() { that.removeClass("selected"); }, 800);
          }
        }
      });
    }
  });

  onPrononciationPlaybackComplete();

  // Add WordClick spans
  $("div.pos > div").not("div.hw,div.pron").each( function() {
   function applyWordClick(node) {
     var skip = 0;
     // Process text nodes except those that are embedded inside of an A tag
     if ((node.nodeType == 3) && (node.parentNode.tagName != 'A')) {
       // We are in a text node

       // Find first alphanumeric string including a hyphen ([a-zA-Z0-9\-]) with at least 4 characters
       var matched = node.data.match(/[a-zA-Z0-9\-']{4,}/);
       if (matched)
       {
         // Wrap word in a <span class="wc">
         var pos = matched.index;
         var spannode = document.createElement('span');
         spannode.className = 'wc';
         var middlebit = node.splitText(pos);
         var endbit = middlebit.splitText(matched[0].length);
         var middleclone = middlebit.cloneNode(true);
         spannode.appendChild(middleclone);
         middlebit.parentNode.replaceChild(spannode, middlebit);
         skip = 1;
       }
     }
     else if (node.nodeType == 1 && node.childNodes && !/(script|style)/i.test(node.tagName)) {
       // recurse into nodes except those matching script or style nodes
       for (var i = 0; i < node.childNodes.length; ++i) {
         i += applyWordClick(node.childNodes[i]);
       }
     }
     return skip;
   }
   applyWordClick(this); 
  });

  // Process WordClick clicks
  $('span.wc').bind('click', function() {
    console.log("window.getSelection().toString() = " + window.getSelection().toString());
    if (window.getSelection().toString().length > 0) return false;

    var that = $(this);
    // Apply a temporary style to selected term for word click
    that.addClass("selected");
    
    window.location.href = "x-mw://lookup/" + that.text();
    console.log("Try to open: " + "x-mw://lookup/" + that.text());

    // After a small delay, unselect the term
    setTimeout(function() { that.removeClass("selected"); }, 800);
    
    // Stop propogation and prevent default
    return false;
  });

  if (!wotd) {
    // Add Favorite stars
    $("div.hw").append('<a href="x-mw://star" class="favorite"></a>');
  }
});

function mwAddStarThes() { mwAddStar("thesaurus"); }
function mwAddStarDef() { mwAddStar("definition"); }
function mwAddStar(divId) {
  if (learners) {
    $(".favorite").addClass("starred");
  } else {
    $(".favorite", "div#" + divId).addClass("starred");
  }
}

function mwRemoveStarThes() { mwRemoveStar("thesaurus"); }
function mwRemoveStarDef() { mwRemoveStar("definition"); }
function mwRemoveStar(divId) {
  if (learners) {
    $(".favorite").removeClass("starred");
  } else {
    $(".favorite", "div#" + divId).removeClass("starred");
  }
}

function showDictionary() {
  window.scrollTo(0, 0);
  $("#definition").show();
  $("#thesaurus").hide();
  console.log("Dictionary shown");
}

function showThesaurus() {
  window.scrollTo(0, 0);
  $("#definition").hide();
  $("#thesaurus").show();
  console.log("Thesaurus shown");
}

function onPrononciationPlaybackComplete() {
  $(".au").each(function(index) {
    var audioBtn = $(this)
    audioBtn.removeClass("playing")
    audioBtn.unbind("click");
    audioBtn.click(function(e) {        
      e.preventDefault();
      
      audioBtn.toggleClass("playing");
      audioBtn.unbind("click");
      audioBtn.click(function(e) {
        e.preventDefault();
      });

      window.location.href = "x-mw://audio/" + $(this).attr("href");        
    });
  });
}

function removeImages(hide) {
  $("div.pos").each(function() {
    var hw = $("div.hw", this);
    var art = $("div.art", this);
    if (hide && art.length) {
      $(art).find('img').each(function(){
        var artLink = $('<a class="art-link"></a>');
        var url = "x-mw://gallery/?image=" +$(this).attr('src') + "&amp;caption=";
        artLink.attr('href', url);

        hw.append(artLink);
      });
      art.hide();
    } else {
      art.remove();
    }
  });
}
